; ****************************************************************************
;
;    *** disclaimer goes here ***
;
; ****************************************************************************
;
;   Developer: Perry S. Morris (Relvinian)
;     Created: Nov 7th, 2006  4:15pm
;
; ****************************************************************************
;
; HISTORY:
;    Version 1.0:
;       - Initial file creation
;       - _asm_AllocMemory@4 proc export added
;
; ============================================================================
.486
.model flat

option readonly
option casemap:none
option epilogue:none
option prologue:none
option proc:export


; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
include Kernel32.inc


; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------


; ----------------------------------------------------------------------------
; Memory library specific defines and global variables
; ============================================================================
extern hProcessHeap:near


.code

; ----------------------------------------------------------------------------
; prototype:
;   void* __stdcall asm_AllocMemory(const size_t tSize);
;
; description:
;   this function will return memory which is aligned on a 16-byte memory
;   segment. having memory aligned gives a performance increase on most
;   processors plus a lot of the newer instruction sets (MMX, SSE, SSE2,
;   SSE3 and 3DNow will throw exceptions if the memory isn't properly
;   aligned. all memory is zeroed out.
;
; returns:
;   Success: Pointer to the new memory block
;    Failed: NULL
; ============================================================================
align 4
_asm_AllocMemory@4 proc
	; get our allocation size parameter
    mov eax, [hProcessHeap]
	mov ecx, [esp + 4]
	
	; Adjust the size of the allocation so we can align it on the correct
	; memory segment offset. we are also going to add some extra space for
	; the structure which keeps the 'real' allocation information. The value
	; 23 is 8 bytes to store the 'real' memory pointer plus four bytes of 
	; validation checking data. The other 15 bytes are to allow for aligning the
	; returned pointer on a 16-byte memory offset.
	add ecx, 23

	; get our requested memory from the OS
	push ecx                ; size of memory block to allocate
	push HEAP_ZERO_MEMORY   ; zero out the memory block
	push [eax]              ; heap process handle
	call HeapAlloc
	test eax, eax           ; did allocation succeed?
	je short allocDone
	
	; we have, so lets update some internal references and return
	; the correct alignment position into the memory.
	; aligned_ptr = ((memory_ptr + ALIGNMENT_ADJUST + sizeof(ABH)) & ~ALIGNMENT_ADJUST);
	mov ecx, eax            ; keep original address of block
	add eax, 23             ; ALIGNMENT_ADJUST + sizeof(ABH) 
	and eax, 0fffffff0h     ; align return address on 16-byte boundary

	mov [eax-8], ecx        ; store original address for later freeing
	mov dword ptr [eax-4], 0fffefdfch ; set validation check data
	
allocDone:
	ret 4
_asm_AllocMemory@4 endp

END