; ****************************************************************************
;
;    *** disclaimer goes here ***
;
; ****************************************************************************
;
;   Developer: Perry S. Morris (Relvinian)
;     Created: Nov 7th, 2006  4:15pm
;
; ****************************************************************************
;
; HISTORY:
;    Version 1.0:
;       - Initial file creation
;       - _asm_AllocInitialization@0 proc export added
;
; ============================================================================
.486
.model flat

option readonly
option casemap:none
option epilogue:none
option prologue:none
option proc:export


; ----------------------------------------------------------------------------
; System Includes
; ----------------------------------------------------------------------------
include Kernel32.inc


; ----------------------------------------------------------------------------
; Library includes
; ----------------------------------------------------------------------------


; ----------------------------------------------------------------------------
; static data to this module (.asm file)
; ----------------------------------------------------------------------------
.const

align 16
IF _UNICODE eq 1	; same definition as the MFC library (windows API uses UNICODE)
    kernel32W    db 'k',0,'e',0,'r',0,'n',0,'e',0,'l',0,'3',0,'2',0,'.',0,'d',0,'l',0,'l',0, 0, 0
    kernel32DLL  equ <kernel32W>
ELSE
    kernel32A    db 'kernel32.dll', 0
    kernel32DLL  equ <kernel32A>
ENDIF

align 16
    HeapSetInformationProc db 'HeapSetInformation', 0


.data
align 4
    hProcessHeap dd 0

public hProcessHeap     ; set public for other modules to reference

.code

; ----------------------------------------------------------------------------
; Prototype:
;   void __stdcall asm_AllocInitialization();
;
; Description:
;   Initializes the memory routines.
;
;   NOTE #1:
;       This function must be called before any asm_AllocMemory() or
;       asm_ReAllocMemory() are called. If this function isn't, the memory
;       routines will generate an exception and will crash the application.
;
;   NOTE #2:
;       If you are using the ASMRTL10.DLL and not just manually including an
;       individual .asm file, the initialization of the .DLL will handle
;       calling this function so you do not have to worry about it in your
;       application code. Otherwise, you must call this function on your
;       own.
;
; Returns:
;   (none)
; ============================================================================
align 4
_asm_AllocInitialization@0 proc
    call GetProcessHeap     ; get process heap handle
    mov [hProcessHeap], eax ; save for future module use

    ; this next section of code will check to see if we can use the low-
    ; fragmentation routines on Windows/XP or higher. We are going to
    ; dynamically load the kernel32.dll file and use GetProcAddress() to
    ; see if the function 'HeapSetInformation' is available for calling.
    push offset kernel32DLL     ; name of dll file
    call LoadLibrary
    test eax, eax               ; did the .dll file load?
    je short initFinished

    push eax                    ; save handle so we can unload it
    push offset HeapSetInformationProc   ; name of procedure in .dll file
    push eax                    ; handle of .dll file
    call GetProcAddress         
    test eax, eax               ; is that procedure available?
    je short unloadDLL

    sub esp, 4                  ; space for a DWORD variable
    mov edx, esp
    mov dword ptr [edx], 2      ; only support value by OS for LFH

    ; NOTE:
    ;   HeapSetInformation *always* fails in debug mode or on OSes that have
    ;   the debug version of the system files. You just have to "believe" in
    ;   the OS call to be successfull on release builds. :-(
    push sizeof DWORD           ; size of variable
    push edx                    ; variable on stack
    push HeapCompatibilityInformation
    call eax                    ; proc address

unloadDLL:
    ; we have already pushed the handle, so, just make the call to unload it.
    call FreeLibrary

initFinished:
	ret 0
_asm_AllocInitialization@0 endp

END